import re
import os
import json
import traceback
from collections import OrderedDict
import requests
from .logger import lg
from ..Cerebro import Cerebro
cerebro = Cerebro()

'''
实现的功能：
    采集步骤顺序运行方法
    子类创建时，生成缓存文件，缓存文件对象作为类属性，传递给子类
    将请求类集成到基类中，每次请求发生时

'''


class TestBiz(type):
    def __init__(self, *args, **kwargs):
        if args[1]:  # 子类为业务类
            cls_name = args[0]
            base_dir = os.path.dirname(os.path.abspath(__file__))
            # 在测试项目中创建 .cache 文件夹， 存放用例流程缓存文件
            if not os.path.exists('.cache'):
                os.mkdir('.cache')
            # 生成用例的缓存文件
            cache_file = os.path.join('.cache', f'.{cls_name}.json')
            if os.path.exists(cache_file):
                os.remove(cache_file)
            self.cache_file = cache_file
            # 创建一个requests.Session() 用于请求, 保存信息
            self.session = requests.Session()
            # 创建用例流程json
            self.flow_json = OrderedDict()
            self.flow_json['INFO'] = {
                '测试环境': cerebro.test_environment
            }

        super().__init__(*args, **kwargs)


class RealiBoxTestBiz(metaclass=TestBiz):
    def __init__(self):
        self.flow_json['STEPS'] = list()  # 步骤列表汇总
        self.current_step_name = ''  # 当前运行的步骤函数名称
        self.current_step_doc = ''  # 当前运行的步骤函数__doc__
        self.V = type('V', (), {})  # 用于存放用例过程中的变量
        self.assert_expr_fn_maps = {  # 断言解析字符串与断言方法匹配
            '^GT\s(.*?)': 'assert_gt',
            '^LT\s(.*?)': 'assert_ls',
            '^GTE\s(.*?)': 'assert_gte',
            '^LTE\s(.*?)': 'assert_lte',
            r'^=\s(.*?)': 'assert_equal',
            r'^T\s(.*?)': 'assert_true',
            r'^LEN\s(.*?)': 'assert_len',
            r'^LEN_GT\s(.*?)': 'assert_len_gt',
            r'^LEN_GTE\s(.*?)': 'assert_len_gte',
            r'^LEN_LT\s(.*?)': 'assert_len_lt',
            r'^LEN_LTE\s(.*?)': 'assert_len_lte',
        }

    def run(self):
        # 收集测试步骤函数
        #  首先获取到子类
        cls = self.__class__
        #  获取子类属性
        attrs = cls.__dict__.keys()
        #  提取步骤函数名称
        steps = filter(lambda x: x if re.match(r'step', x) else None, attrs)
        # 按顺序遍历函数运行
        #   生成测试用例对象
        case_object = cls()
        try:
            #   遍历步骤运行
            for index, step in enumerate(steps):
                # 为用例对象创建多个属性记录数据， 在多个地方进行填充，
                # 最后步骤执行完毕后，汇总写入用例json中
                setattr(case_object, 'step_requests_list', list())  # 初始化每个步骤的请求列表
                setattr(case_object, 'step_assertions_list', list())  # 初始化每个步骤的断言列表
                # 步骤函数对象
                func = getattr(case_object, step)
                case_object.current_step_name = func.__name__  # 设置当前的 步骤函数名称
                case_object.current_step_doc = func.__doc__ or ''  # 设置当前的步骤 函数 __doc__
                case_object.current_step_index = index  # 当前执行第几步

                # 日志输出用例变量
                attrs = case_object.V.__dict__
                lg.info('*'*40 + case_object.current_step_name + '*'*40)
                for k, v in attrs.items():
                    if not re.match('__', k):
                        lg.info(f'变量: {k} - 值: {v}')

                try:
                    func()  # 执行步骤函数

                except Exception as e:
                    # 用例步骤出现异常情况
                    #   1、获取异常的用例类-cls、步骤名称-step，异常信息
                    case_object.flow_json.update({
                        'STEPS_ERROR': {
                            'step_name': case_object.current_step_name,
                            'step_doc': case_object.current_step_doc,
                            'msg': str(e),
                            'traceback': traceback.format_exc()
                        }
                    })
                    raise
                finally:
                    # 用例步骤列表flow_json.STEPS中追加一个字典，预设desc
                    # 为步骤描述（步骤函数__doc__）,requests为请求列表
                    case_object.flow_json['STEPS'].append(
                        {
                            case_object.current_step_name: {
                                'desc': case_object.current_step_doc,
                                'requests': case_object.step_requests_list,
                                'assertions': case_object.step_assertions_list,
                            }
                        }
                    )

        except Exception:
            raise

        finally:
            if cerebro.is_save_case_json_locally:
                # 执行io写入json文件
                # 用例对象中的 步骤缓存文件
                cache_file = self.cache_file
                with open(cache_file, 'w') as f:
                    json.dump(case_object.flow_json, f, ensure_ascii=False)